import json
from django.shortcuts import render, reverse, HttpResponseRedirect, HttpResponse
from django.views.generic import ListView, DetailView, CreateView
from django.db.models import Q
from django.contrib.auth.decorators import login_required
from django.utils.decorators import method_decorator
from django.db.models import Max, Min

from .models import News, WpEsiMedia, WpEsiTag
from .forms import NewsForm


@method_decorator(login_required, name='dispatch')
class EditNewsView(DetailView):
    template_name = 'edit_news.html'

    def test_func(self):
        return self.request.user.groups.filter(name='admin').exists()

    def get(self, request, *args, **kwargs):
        news = News.objects.get(id=kwargs.get('id'))
        return render(request, self.template_name, {'news': news})

    def post(self, request, *args, **kwargs):
        form = NewsForm(request.POST)
        if form.is_valid():
            form.update()

        return HttpResponseRedirect('/news')


class ViewNewsView(DetailView):
    template_name = 'view_news.html'

    def get(self, request, *args, **kwargs):
        news_id = kwargs.get('id')
        news = News.objects.get(id=news_id)

        return render(request, self.template_name, {'news': news})

    def post(self, request, *args, **kwargs):
        delete_news = request.POST.get('delete')
        export_news = request.POST.get('export')

        if delete_news:
            News.objects.filter(id=delete_news).update(is_accepted=True)
            return HttpResponseRedirect(reverse('view_news', kwargs={'id': delete_news}))

        if export_news:
            news = News.objects.get(id=export_news)
            news.export_news()
            return HttpResponseRedirect(reverse('view_news', kwargs={'id': export_news}))


@method_decorator(login_required, name='dispatch')
class AddNewsView(CreateView):
    template_name = 'add_news.html'

    def get(self, request, *args, **kwargs):
        return render(request, template_name=self.template_name)

    def post(self, request, *args, **kwargs):
        form = NewsForm(request.POST)
        if form.is_valid():
            form.create()

        return HttpResponseRedirect(reverse('list_news'))


@method_decorator(login_required, name='post')
class ListNewsView(ListView):
    context_object_name = 'news'
    queryset = News.objects.filter(is_accepted=False).order_by('-id')
    template_name = 'list_news.html'
    sort_fields = ('media_id', 'type_id', 'region_id', 'publish_date', 'title')

    def post(self, request, *args, **kwargs):
        if request.is_ajax():
            data = json.loads(request.body.decode('utf-8'))
            type_request = data['type']
            ids = data['id']
            if type_request in ['delete', 'deleteAll']:
                self._delete(ids)
            if type_request == 'edit':
                return HttpResponse({'ids': ids})

            if type_request == 'exportAll':
                for i in News.objects.filter(pk__in=ids):
                    i.export_news()
            return HttpResponse(200)
        return HttpResponse(200)

    def get_queryset(self):
        data = self.request.GET

        media = data.get('media')
        from_date = data.get('from_date')
        to_date = data.get('to_date')
        tag = data.get('tag')
        search = data.get('search')
        order_by = data.get('order_by')

        if tag:
            self.queryset = self.queryset.filter(tags_id__contains='{},'.format(WpEsiTag.objects.get(name=tag).id))
        if data.get('reverse'):
            self.queryset = self.queryset.reverse()
        if order_by in self.sort_fields:
            self.queryset = self.queryset.order_by(order_by)
        if search:
            self.queryset = self.queryset.filter(Q(title__icontains=search) | Q(description__icontains=search))
        if from_date:
            self.queryset = self.queryset.filter(publish_date__gte=from_date)
        if to_date:
            self.queryset = self.queryset.filter(publish_date__lte=to_date)
        if media and media != 'All':
            self.queryset = self.queryset.filter(media_id=WpEsiMedia.objects.get(name=media).pk)

        return self.queryset

    def get_context_data(self, **kwargs):
        context = super(ListNewsView, self).get_context_data(**kwargs)
        context['media'] = WpEsiMedia.objects.all()
        min_max_date = News.objects.aggregate(Min('publish_date'), Max('publish_date'))
        min_max_date['publish_date__min'] = str(min_max_date['publish_date__min'].date())
        min_max_date['publish_date__max'] = str(min_max_date['publish_date__max'].date())
        context['date'] = min_max_date
        return context

    def _delete(self, id):
        if isinstance(id, list):
            News.objects.filter(id__in=id).update(is_accepted=True)
        else:
            News.objects.filter(id=id).update(is_accepted=True)
